#include <xsTypes.h>
#include <functionMap.h>
#include <stlToCArrays.h>
#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>


PROTOCCALLSFSUB16(NEISPEC,neispec,INT,FLOATV,INT,FLOATV,INTV,INTV,INT,FLOATV,FLOAT,FLOATV,INT,FLOATV,INTV,LOGICAL,STRING,FLOATV)

#define NEISPEC(ntemp,temp,nion,ionfrac,ionel,ionstage,ne,ear,z,linesh,nabun,abund,abunel,qcalc,callr,photar) \
           CCALLSFSUB16(NEISPEC,neispec,INT,FLOATV,INT,FLOATV,INTV,INTV,INT,FLOATV,FLOAT,FLOATV,INT,FLOATV,INTV,LOGICAL,STRING,FLOATV, \
           ntemp,temp,nion,ionfrac,ionel,ionstage,ne,ear,z,linesh,nabun,abund,abunel,qcalc,callr,photar) 

extern "C" void initnei_(int& nionp, int& nzmax);

void ionsneqs(float* tmp, int ntmp, float* tau, int n, float* weight, 
                int nzmax, int nionp, float* fout, int* ionel, 
                int* ionstage);
                
void xsneq(const RealArray& energyArray, const RealArray& params,
       int spectrumNumber, RealArray& flux, RealArray& fluxErr, 
       const string& initString)
{
   using namespace XSutility;

   //
   //  aug 95   Non-equlibrium ionization models with constant electron
   //           temperature
   //		additive model for XSPEC to use with general models.
   //		parameters (params):
   //           (0):    temperature t(keV)
   //           (1):    hydrogen density in cm^-3
   //		(2..13): abundances of He,C,N,O,Ne,Mg,Si,S,Ar,Ca,Fe,Ni
   //                                  with respect to solar values
   //           (14):   ionization timescale tau (cm^-3 s^-1)
   //		(15):	redshift z
   //
   //           Parameter (1) should be usually set to one. Its only use
   //           in the code is to set hydrogen abundance to zero; in this
   //           case this parameter should be set to zero. It is not the
   //           density parameter.
   //

   // Conversion factors.
   const Real keV = 1.60219e-9;
   const Real boltz = 1.38062e-16;
   const Real tconv = keV/boltz;
   
   static bool isFirst = true;
   static int nIonp=0;
   static int nZmax=0;
   static auto_array_ptr<float> apEi(0);
   static auto_array_ptr<int> apIonEl(0);
   static auto_array_ptr<int> apIonStage(0);
   if (isFirst)
   {
      initnei_(nIonp, nZmax);
      apEi.reset(new float[nIonp]);
      apIonEl.reset(new int[nIonp]);
      apIonStage.reset(new int[nIonp]);
      isFirst = false;
   }
   
   const int nAbun = 13;
   
   Real t = tconv*params[0];
   Real tau = params[nAbun+1];
   Real z = params[nAbun+2];
   
   bool doRecalc = false;
   static Real tprev = 0.0;
   static Real taup = 0.0;
   const int NTEMPR = 2;
   if (t != tprev || tau != taup)
   {
      // Ionic concentrations for solar abundances.
      
      float tempr[NTEMPR];
      tempr[0] = static_cast<float>(t);
      tempr[1] = static_cast<float>(t);
      float weight=1.0;
      float ftau = static_cast<float>(tau);
      ionsneqs(tempr, NTEMPR, &ftau, 1, &weight, nZmax, nIonp, 
                apEi.get(), apIonEl.get(), apIonStage.get());      
      tprev = t;
      taup = tau;
      doRecalc = true;
   }
   
   // Calculate the spectrum for this temperature and ion fractions.
   
   float abund[nAbun];
   int abundEl[nAbun];
   static const int atomZ[nAbun]={1, 2, 6, 7, 8, 10, 12, 14, 16, 18, 20, 26, 28};
   
   abund[0] = params[1];
   abundEl[0] = atomZ[0];
   for (int i=1; i<nAbun; ++i)
   {
      abund[i] = params[i+1];
      abundEl[i] = atomZ[i];
   }
   float nc_par0 = params[0];
   int nE = energyArray.size()-1;
   float linesh[2];
   char funcName[] = "xsneq";
   float fz = static_cast<float>(z);

   float *ear=0, *pars=0, *photar=0, *photer=0;
   XSFunctions::stlToFloatArrays<float>(energyArray, params, flux, fluxErr,
           ear, pars, photar, photer);
   auto_array_ptr<float> apEar(ear);
   auto_array_ptr<float> apPars(pars);
   auto_array_ptr<float> apPhotar(photar);
   auto_array_ptr<float> apPhoter(photer);
   
   NEISPEC(1, &nc_par0, nIonp, apEi.get(), apIonEl.get(), apIonStage.get(), 
        nE, ear, fz, linesh, nAbun, abund, abundEl, doRecalc, funcName, photar);
        
   XSFunctions::floatFluxToStl<float>(photar, photer, nE, false, flux, fluxErr);       
}
